# Amethyst - An experimental Minecraft server written in Elixir.
# Copyright (C) 2024 KodiCraft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

defmodule Amethyst.DataGenerator do
  @moduledoc """
  This module allows Amethyst to download the vanilla Minecraft server and generate
  necessary data files.
  """
  @spec get_version_meta(String.t() | :latest) :: {:ok, map()} | {:error, term}
  def get_version_meta(version) do
    version = if version == :latest, do: get_latest_version(), else: {:ok, version}
    case version do
      {:ok, version} ->
        case Req.get(Application.fetch_env!(:amethyst, :mojang_game_meta)) do
          {:ok, %{body: %{"versions" => versions}}} ->
            case Enum.find(versions, fn %{"id" => id} -> id == version end) do
              nil -> {:error, :version_not_found}
              version_meta -> {:ok, version_meta}
            end
          {:error, err} -> {:error, err}
        end
      {:error, err} -> {:error, err}
    end
  end

  def get_latest_version() do
    case Req.get(Application.fetch_env!(:amethyst, :mojang_game_meta)) do
      {:ok, %{body: %{"latest" => %{"release" => release}}}} -> {:ok, release}
      {:error, err} -> {:error, err}
    end
  end
end
